/*
 * ir_tx.c
 *
 *  Created on: 2019/01/27
 *      Author: http://triode.dip.jp/honkytonk/
 *      Copyright (c) honkytonk All right reserved.
 */

#define _IR_TX_C_
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include "main.h"
#include "ir_tx.h"
#include "stm32f1xx_hal.h"

extern TIM_HandleTypeDef htim4;
extern TIM_HandleTypeDef htim3;

void set_ir_carrier_frq(uint16_t carrier_freq, uint16_t duty)
{
	#define TIMER_CLK_FREQ 72000000L

	uint32_t counter_period;
	uint32_t duty_count = 0;

	counter_period = TIMER_CLK_FREQ / (htim4.Init.Prescaler +1);
	counter_period *= 10;

	counter_period /= carrier_freq;
	counter_period += 5;
	counter_period /= 10;

	duty_count = counter_period * duty +50;
	duty_count /= 100;

	TIM4->ARR = counter_period -1;
	TIM4->CCR1 = duty_count;

}

void startIrSendTimer()
{
	  TIM4->CCER &= ~1;
	  HAL_TIM_PWM_Start(&htim4,TIM_CHANNEL_1);
	  TIM3->CCR1 = 10000;
	  TIM4->CCER &= ~1;

	  HAL_TIM_OC_Start_IT(&htim3, TIM_CHANNEL_1);
}

void initIrSendTimer()
{
	  TIM4->CCER &= ~1;
	  HAL_TIM_PWM_Start(&htim4,TIM_CHANNEL_1);
	  TIM3->CCR1 = 10000;
	  TIM4->CCER &= ~1;

	  HAL_TIM_OC_Start_IT(&htim3, TIM_CHANNEL_1);
}

void makeIrStreamTvFormat(irStream *stream, uint16_t irCode)
{
	uint8_t irCustom;
	uint8_t irOpCode;
	uint8_t b = 0;
	uint32_t time = 0;

	irCustom = (irCode & 0xff00) >> 8;
	irOpCode = irCode & 0x00ff;

	uint8_t mask = 0x01;

	for(int i = 0; i< 8; i++){
		if(irCustom & (mask << i)){ // 1
			bitStream[b].period = 300;
			bitStream[b++].isMark = 1;
			bitStream[b].period = 2140;
			bitStream[b++].isMark = 0;

			time += 300 + 2140;
		}
		else{
			bitStream[b].period = 300;
			bitStream[b++].isMark = 1;
			bitStream[b].period = 920;
			bitStream[b++].isMark = 0;

			time += 300 + 920;
		}
	}

	mask = 0x01;
	for(int i = 0; i< 8; i++){
		if(irOpCode & (mask << i)){ // 1
			bitStream[b].period = 300;
			bitStream[b++].isMark = 1;
			bitStream[b].period = 2140;
			bitStream[b++].isMark = 0;

			time += 300 + 2140;
		}
		else{
			bitStream[b].period = 300;
			bitStream[b++].isMark = 1;
			bitStream[b].period = 920;
			bitStream[b++].isMark = 0;

			time += 300 + 920;
		}
	}

	bitStream[b].period = 300;
	bitStream[b++].isMark = 1;

	time += 300;
	stream->time = time;

	bitStream[b].period = 53700 - time;
	bitStream[b++].isMark = 0;

	stream->bitStream = bitStream;
	stream->size = b;
	stream->repeat = 1;
}


void ir_send(uint16_t irCode)
{

	makeIrStreamTvFormat(&irBitStream, irCode);
	set_ir_carrier_frq(38000, 30);
	irBitStream.sendbit = 0;
	restartIrSend();
}

void restartIrSend()
{
	irBitStream.sendbit = 0;
}

void HAL_TIM_OC_DelayElapsedCallback(TIM_HandleTypeDef *htim)
{

	if(htim->Instance != TIM3) return;

	  if(irBitStream.repeat == 0){
		  TIM4->CCER &= ~1; // Disable PWM output
		  return;
	  }

	  switch(irBitStream.bitStream[irBitStream.sendbit].isMark){
	  case 0:
		  TIM4->CCER &= ~1;
		  break;
	  case 1:
		  TIM4->CCER |= 0x0001;
		  break;
	  }

	  TIM3->CCR1 = irBitStream.bitStream[irBitStream.sendbit++].period;
	  TIM4->CNT = 0;
	  TIM3->CNT = 0;


	  if(irBitStream.sendbit == irBitStream.size){
		  irBitStream.repeat --;
		  irBitStream.sendbit = 0;
	  }

	  HAL_TIM_OC_Start_IT(htim, TIM_CHANNEL_1);
#if 1
	  HAL_GPIO_TogglePin(GPIOC, LED_GREEN_Pin);
//	  HAL_GPIO_TogglePin(GPIOB, LED_BlackPill_Pin);
#endif
}
