/*
 * tm1638.cpp
 *
 *  Created on: May 12, 2019
 *      Author: http://triode.dip.jp/honkytonk/
 *      Copyright (c) honkytonk All right reserved.
 */

#include <stdint.h>
#include "tm1638.h"

void TM1638::setClockPort(gpio io)
{
	clk = io;
}

void TM1638::setStrobePort(gpio io)
{
	stb = io;
}

void TM1638::setDioPort(gpio io)
{
	dio = io;
}

void shortWait(uint16_t wait)
{
	wait *= 1;

	do{

		wait--;

	}while(wait);
}

void TM1638::sendShiftData(uint8_t *data, uint8_t len)
{
	HAL_GPIO_WritePin(dio.group, dio.port, GPIO_PIN_SET);
	HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_SET);
	shortWait(1);

	for(uint16_t i = 0; i < len; i++){
		uint16_t mask = 0x01;

		for(uint8_t j = 0; j < 8; j++){
			HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_RESET);

			if(data[i] & mask){
				HAL_GPIO_WritePin(dio.group, dio.port, GPIO_PIN_SET);
			}
			else{
				HAL_GPIO_WritePin(dio.group, dio.port, GPIO_PIN_RESET);
			}

			mask <<= 1;
			shortWait(1);
			HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_SET);
			shortWait(2);
		}
	}

	HAL_GPIO_WritePin(dio.group, dio.port, GPIO_PIN_SET);
	HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_SET);
}

void TM1638::sendData(uint8_t *data, uint8_t len)
{
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_SET);
	shortWait(1);

	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_RESET);
	this->sendShiftData(data, len);
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_SET);
}

uint8_t TM1638::readByte(void)
{
	uint8_t readData = 0;
	uint8_t mask = 0x01;

	HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_SET);
	HAL_GPIO_WritePin(dio.group, dio.port, GPIO_PIN_SET);

	for(uint8_t i = 0; i < 8; i++){
		HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_RESET);
		shortWait(1);

		HAL_GPIO_WritePin(clk.group, clk.port, GPIO_PIN_SET);
		if(HAL_GPIO_ReadPin(dio.group, dio.port)){
			readData |= mask << i;
		}
	}
	return readData;
}

void TM1638::readKey(void)
{
	uint8_t readKeyCmd = 0b01000010;

	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_SET);
	shortWait(1);
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_RESET);

	this->sendShiftData(&readKeyCmd, 1);
	shortWait(2);

	for(uint8_t i = 0; i < 4; i++){
		*(this->keyReadData + i) = this->readByte();
	}

	shortWait(2);
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_SET);

	uint8_t mask1 = 0x01, mask2 = 0x10;
	this->keyBit = 0;

	// Convert read key data to KEY POSITION
	for(uint8_t i = 0; i < 4; i++){
		uint8_t key = this->keyReadData[i];

		if(key & 0x01){ // K3 & KS1, KS3, KS5, KS7
			this->keyBit |= mask1;
		}
		mask1 <<= 1;

		if(key & 0x10){ // K3 & KS2, KS4, KS6, KS8
			this->keyBit |= mask2;
		}
		mask2 <<= 1;
	}
}

uint8_t TM1638::getKeyBit(void)
{
	return this->keyBit;
}

void TM1638::setLed(uint8_t led)
{
	uint8_t mask = 0x01;

	for(uint8_t i = 0; i < 8; i++){

		if(led & (mask << i)){
			*(this->segmentData + i*2 +1) = 0x01;
		}
		else{
			*(this->segmentData + i*2 +1) = 0;
		}
	}
}

void TM1638::sendDisplayRegister(uint8_t adrs)
{
	uint8_t cmd1 = 0b01000000;
	this->sendData(&cmd1, 1);

	if(adrs > 0x0f) adrs = 0x0f;

	shortWait(1);
	uint8_t cmd2 = 0b11000000|(adrs & 0x0f);
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_RESET);
	shortWait(1);
	this->sendShiftData(&cmd2, 1);

	uint8_t len = 16 - adrs;

	this->sendShiftData(this->segmentData, len);
	HAL_GPIO_WritePin(stb.group, stb.port, GPIO_PIN_SET);
}

void TM1638::keyTest(void)
{
	for(uint8_t i = 0; i < 4; i++){
		*(this->segmentData + i*4) = numTo7Seg[*(this->keyReadData + i)&0x0f];
		*(this->segmentData + i*4 +2) = numTo7Seg[(*(this->keyReadData + i)&0xf0)>>4];
	}
}

void TM1638::clearDigit(void)
{
	for(uint8_t i = 0; i < sizeof(segmentData)/sizeof(uint8_t); i +=2){
		*(this->segmentData + i) = 0x00;
	}
}

void TM1638::clearDisplayRegister(void)
{
	for(uint8_t i = 0; i < sizeof(segmentData)/sizeof(uint8_t); i++){
		*(this->segmentData + i) = 0x00;
	}

	this->sendDisplayRegister(0);
}

void TM1638::setDegit(uint8_t digit, uint8_t num, uint8_t dot)
{
	/*
	 * dot
	 *  0x00: both dot and LED are OFF
	 *  0x01: dot is ON
	 *  0x02: LED is ON
	 *  0x03: both dot and LED are ON.
	 */

	uint8_t* data;
	data = this->segmentData + (digit *2);

	if(num > 0x0f){num = sizeof(numTo7Seg)/sizeof(uint8_t)-1; }
	*data = numTo7Seg[num];

	if(dot & 0x01){ // dot
		*data |= 0x80;
	}

	if(dot & 0x02){ // LED
		*(data +1 ) |= 0x01;
	}
}

void TM1638::setDecimalNumber(uint32_t num)
{
	uint32_t div = 10000000;
	bool zeroEnb = false;

	this->clearDigit();
	if(num > 99999999) {num = 99999999;}

	for(uint8_t col = 0; col < 8 ; col ++){
		uint8_t s = (num / div);

		if(s || zeroEnb){
			this->setDegit(col, s, 0x00);
			zeroEnb = true;
		}
		else{
			this->setDegit(sizeof(numTo7Seg)/sizeof(uint8_t) -2, s, 0x00);
		}
		num = num % (s * div);
		div /= 10;
	}
}

void TM1638::setAsciiStr(uint8_t place, uint8_t* str)
{
	uint8_t* data;
	uint8_t len = 0;
	uint8_t dot = 0;


	for(len = 0; len < 255; len++){
		if(str[len] == '.'){ dot++;}
		if(str[len] == 0x00){ break;}
	}

	if(len > 8 +dot){ len = 8 +dot;}

	for(uint8_t i = 0, j = 0; i < len && j < 8; ){
		data = this->segmentData + (place + j)*2;

		if(str[i] < 0x80){
			if(str[i] != '.'){
				*data = asciiTo7Seg[str[i]];
				j++;

				if((i+1 < len) && (str[i+1] == '.')){
					*data |= 0x80;
					i++;
				}
			}
			else if(str[i] == '.'){
				*data |= 0x80;
				j++;
			}
		}
		i++;
	}
}


void TM1638::sendDisplayCtrlCmd(uint8_t mode, uint8_t duty)
{
	uint8_t cmd = 0b10000000;

	if(mode == 1){ // Display ON
		cmd |= 0b00001000;
	}

	// Brightness
	if(duty > 7){duty = 7;}

	cmd |= duty;

	this->sendData(&cmd, 1);
}


